@@ -3,10 +3,18 @@
 #include "pdl.h"      /* Data structure declarations */
 #include "pdlcore.h"  /* Core declarations */
 
-#define VTABLE_OR_DEFAULT(what, trans, func, default_func) \
-  what(PDL_err, ((trans)->vtable->func \
-    ? (trans)->vtable->func \
-    : pdl_ ## default_func)(trans))
+#define VTABLE_OR_DEFAULT(what, trans, is_fwd, func, default_func) \
+  do { \
+    PDLDEBUG_f(printf("VTOD call " #func "(%p=%s)\n", trans, trans->vtable->name)); \
+    what(PDL_err, ((trans)->vtable->func \
+      ? (trans)->vtable->func \
+      : pdl_ ## default_func)(trans)); \
+    pdl **pdls = trans->pdls; \
+    PDL_Indx i, istart = is_fwd ? trans->vtable->nparents : 0, iend = is_fwd ? trans->vtable->npdls : trans->vtable->nparents; \
+    for (i = istart; i < iend; i++) \
+      if (pdls[i] && (pdls[i]->state & PDL_BADVAL)) \
+        pdl_propagate_badflag(pdls[i], !!(pdls[i]->state & PDL_BADVAL)); \
+  } while (0)
 
 #define REDODIMS(what, trans) do { \
     if ((trans)->vtable->redodims) \
@@ -23,8 +31,8 @@
       ? (trans)->vtable->redodims \
       : pdl_redodims_default)(trans)); \
   } while (0)
-#define READDATA(trans) VTABLE_OR_DEFAULT(PDL_ACCUMERROR, trans, readdata, readdata_affine)
-#define WRITEDATA(trans) VTABLE_OR_DEFAULT(PDL_ACCUMERROR, trans, writebackdata, writebackdata_affine)
+#define READDATA(trans) VTABLE_OR_DEFAULT(PDL_ACCUMERROR, trans, 1, readdata, readdata_affine)
+#define WRITEDATA(trans) VTABLE_OR_DEFAULT(PDL_ACCUMERROR, trans, 0, writebackdata, writebackdata_affine)
 #define FREETRANS(trans, destroy) \
     if(trans->vtable->freetrans) { \
 	PDLDEBUG_f(printf("call freetrans\n")); \
@@ -37,8 +45,9 @@
 
 extern Core PDL;
 
+pdl_error pdl__make_physvaffine_recprotect(pdl *it, int recurse_count);
 /* Make sure transformation is done */
-pdl_error pdl__ensure_trans(pdl_trans *trans,int what,int *wd)
+pdl_error pdl__ensure_trans(pdl_trans *trans,int what,int *wd, int recurse_count)
 {
 	pdl_error PDL_err = {0, NULL, 0};
 	PDLDEBUG_f(printf("pdl__ensure_trans %p what=%d\n", trans, what));
@@ -46,17 +55,13 @@ pdl_error pdl__ensure_trans(pdl_trans *trans,int what,int *wd)
 	PDL_Indx j, flag=what, par_pvaf=0;
 	pdl_transvtable *vtable = trans->vtable;
 /* Make parents physical */
-	for(j=0; j<vtable->nparents; j++) {
+	for(j=0; j<vtable->npdls; j++) {
 		if(VAFFINE_FLAG_OK(vtable->per_pdl_flags,j))
 			par_pvaf++;
-		PDL_RETERROR(PDL_err, pdl_make_physvaffine(trans->pdls[j]));
+		PDL_RETERROR(PDL_err, pdl__make_physvaffine_recprotect(trans->pdls[j], recurse_count+1));
 	}
-	for(; j<vtable->npdls; j++) {
-		if(VAFFINE_FLAG_OK(vtable->per_pdl_flags,j))
-			par_pvaf++;
-		PDL_RETERROR(PDL_err, pdl_make_physvaffine(trans->pdls[j]));
+	for(j=vtable->nparents; j<vtable->npdls; j++)
 		flag |= trans->pdls[j]->state & PDL_ANYCHANGED;
-	}
 	if (flag & PDL_PARENTDIMSCHANGED) REDODIMS(PDL_RETERROR, trans);
 	for(j=0; j<vtable->npdls; j++)
 		if(trans->pdls[j]->trans_parent == trans)
@@ -66,12 +71,10 @@ pdl_error pdl__ensure_trans(pdl_trans *trans,int what,int *wd)
 		  /* Attention: this assumes affine = p2child */
 		  /* need to signal that redodims has already been called */
 		        trans->pdls[1]->state &= ~PDL_PARENTDIMSCHANGED;
-			PDL_RETERROR(PDL_err, pdl_make_physvaffine(trans->pdls[1]));
+			PDL_RETERROR(PDL_err, pdl__make_physvaffine_recprotect(trans->pdls[1], recurse_count+1));
 			PDL_ACCUMERROR(PDL_err, pdl_readdata_vaffine(trans->pdls[1]));
 		} else
-{
 			READDATA(trans);
-}
 	}
 	for(j=vtable->nparents; j<vtable->npdls; j++) {
 		pdl *child = trans->pdls[j];
@@ -88,31 +91,28 @@ pdl_error pdl__ensure_trans(pdl_trans *trans,int what,int *wd)
 }
 
 pdl *pdl_null() {
-	PDL_Anyval zero = { PDL_D, {.D=0.0} };
 	PDLDEBUG_f(printf("pdl_null\n"));
-	pdl *it = pdl_pdlnew();
-	if (!it) return it;
-	pdl_error PDL_err = pdl_makescratchhash(it, zero);
-	if (PDL_err.error) { pdl_destroy(it); return NULL; }
-	return it;
+	return pdl_pdlnew();
 }
 
 pdl *pdl_scalar(PDL_Anyval anyval) {
 	PDLDEBUG_f(printf("pdl_scalar type=%d val=", anyval.type); pdl_dump_anyval(anyval); printf("\n"););
 	pdl *it = pdl_pdlnew();
 	if (!it) return it;
-	pdl_error PDL_err = pdl_makescratchhash(it, anyval);
-	if (PDL_err.error) { pdl_destroy(it); return NULL; }
+	it->datatype = anyval.type;
 	it->broadcastids[0] = it->ndims = 0; /* 0 dims in a scalar */
-	it->state &= ~(PDL_ALLOCATED|PDL_NOMYDIMS); /* size changed, has dims */
-	it->nvals = 1;            /* 1 val  in a scalar */
+	pdl_resize_defaultincs(it);
+	pdl_error PDL_err = pdl_allocdata(it);
+	if (PDL_err.error) { pdl_destroy(it); return NULL; }
+	it->value = anyval.value;
+	it->state &= ~(PDL_NOMYDIMS); /* has dims */
 	return it;
 }
 
 pdl *pdl_get_convertedpdl(pdl *old,int type) {
 	PDLDEBUG_f(printf("pdl_get_convertedpdl\n"));
 	if(old->datatype == type) return old;
-	pdl *it = pdl_null();
+	pdl *it = pdl_pdlnew();
 	if (!it) return it;
 	pdl_error PDL_err = pdl_converttypei_new(old,it,type);
 	if (PDL_err.error) { pdl_destroy(it); return NULL; }
@@ -300,8 +300,8 @@ void pdl__removetrans_children(pdl *it,pdl_trans *trans)
 
 void pdl__removetrans_parent(pdl *it, pdl_trans *trans, PDL_Indx nth)
 {
-	PDLDEBUG_f(printf("pdl__removetrans_parent(%s=%p): %p %"IND_FLAG"\n",
-	  trans->vtable->name, (void*)trans, (void*)(it), nth));
+	PDLDEBUG_f(printf("pdl__removetrans_parent from %p (%s=%p): %"IND_FLAG"\n",
+	  it, trans->vtable->name, trans, nth));
 	trans->pdls[nth] = 0;
 	if (it->trans_parent == trans) it->trans_parent = 0;
 	it->state &= ~PDL_MYDIMS_TRANS;
@@ -323,7 +323,8 @@ pdl_error pdl_trans_finaldestroy(pdl_trans *trans)
   return PDL_err;
 }
 
-pdl_error pdl_destroytransform(pdl_trans *trans,int ensure,int *wd)
+pdl_error pdl__destroy_recprotect(pdl *it, int recurse_count);
+pdl_error pdl_destroytransform(pdl_trans *trans,int ensure,int *wd, int recurse_count)
 {
 	pdl_error PDL_err = {0, NULL, 0};
 	PDL_TR_CHKMAGIC(trans);
@@ -337,7 +338,7 @@ pdl_error pdl_destroytransform(pdl_trans *trans,int ensure,int *wd)
 			  trans->vtable ? trans->vtable->name : "NULL",
 			  (void*)trans,ensure,ismutual));
 	if(ensure)
-		PDL_ACCUMERROR(PDL_err, pdl__ensure_trans(trans,ismutual ? 0 : PDL_PARENTDIMSCHANGED,wd));
+		PDL_ACCUMERROR(PDL_err, pdl__ensure_trans(trans,ismutual ? 0 : PDL_PARENTDIMSCHANGED,wd, recurse_count+1));
 	pdl *destbuffer[trans->vtable->npdls];
 	int ndest = 0;
 	for(j=0; j<trans->vtable->nparents; j++) {
@@ -364,7 +365,7 @@ pdl_error pdl_destroytransform(pdl_trans *trans,int ensure,int *wd)
 	PDL_ACCUMERROR(PDL_err, pdl_trans_finaldestroy(trans));
 	for(j=0; j<ndest; j++) {
 		destbuffer[j]->state &= ~PDL_DESTROYING; /* safe, set by us */
-		PDL_ACCUMERROR(PDL_err, pdl_destroy(destbuffer[j]));
+		PDL_ACCUMERROR(PDL_err, pdl__destroy_recprotect(destbuffer[j], recurse_count+1));
 	}
 	PDLDEBUG_f(printf("pdl_destroytransform leaving %p\n", (void*)trans));
 	return PDL_err;
@@ -380,11 +381,10 @@ pdl_error pdl_destroytransform(pdl_trans *trans,int ensure,int *wd)
    - allowed to destroy if
       1. a parent with max. 1 backwards propagating transformation
       2. a child with no trans_children
-
   When an ndarray is destroyed, it must tell its trans_children and/or
   parent.
 */
-pdl_error pdl_destroy(pdl *it) {
+pdl_error pdl__destroy_recprotect(pdl *it, int recurse_count) {
     pdl_error PDL_err = {0, NULL, 0};
     int nback=0,nback2=0,nforw=0;
     int nafn=0;
@@ -402,7 +402,6 @@ pdl_error pdl_destroy(pdl *it) {
 	    sv_setiv(it->sv,0x4242);
 	    it->sv = NULL;
     }
-
     /* 1. count the trans_children that do flow */
     PDL_START_CHILDLOOP(it)
 	pdl_trans *curt = PDL_CHILDLOOP_THISCHILD(it);
@@ -415,18 +414,14 @@ pdl_error pdl_destroy(pdl *it) {
 		 * must always be soft-destroyed */
 		if(curt->vtable->npdls > 2) nback2++;
 	}
-
 	if ((curt->flags & PDL_ITRANS_ISAFFINE) && !(curt->pdls[1]->state & PDL_ALLOCATED))
 		nafn ++;
     PDL_END_CHILDLOOP(it)
-
 /* First case where we may not destroy */
     if(nback2 > 0) goto soft_destroy;
     if(nback > 1) goto soft_destroy;
-
 /* Also not here */
     if(it->trans_parent && nforw) goto soft_destroy;
-
 /* Also, we do not wish to destroy if the trans_children would be larger
  * than the parent and are currently not allocated (e.g. lags).
  * Because this is too much work to check, we refrain from destroying
@@ -437,30 +432,28 @@ pdl_error pdl_destroy(pdl *it) {
         PDLDEBUG_f(printf("pdl_destroy not destroying as magic %p\n",(void*)it));
 	goto soft_destroy;
     }
-
     PDL_START_CHILDLOOP(it)
-	PDL_RETERROR(PDL_err, pdl_destroytransform(PDL_CHILDLOOP_THISCHILD(it),1,NULL));
+	PDL_RETERROR(PDL_err, pdl_destroytransform(PDL_CHILDLOOP_THISCHILD(it),1,NULL, recurse_count+1));
     PDL_END_CHILDLOOP(it)
-
     pdl_trans *trans = it->trans_parent;
     if (trans)
         /* Ensure only if there are other children! */
       PDL_RETERROR(PDL_err, pdl_destroytransform(trans,trans->vtable->npdls
-				      - trans->vtable->nparents > 1,NULL));
-
+				      - trans->vtable->nparents > 1,NULL, recurse_count+1));
 /* Here, this is a child but has no children - fall through to hard_destroy */
-
    PDL_RETERROR(PDL_err, pdl__free(it));
    PDLDEBUG_f(printf("pdl_destroy end %p\n",(void*)it));
    return PDL_err;
-
   soft_destroy:
-    PDLDEBUG_f(printf("pdl_destroy may have dependencies, not destroy %p, nba(%d, %d), nforw(%d), tra(%p), nafn(%d)\n",
-				(void*)it, nback, nback2, nforw, (void*)(it->trans_parent), nafn));
+    PDLDEBUG_f(printf("pdl_destroy may have dependencies, not destroy %p, nba(%d, %d), nforw(%d), tra(%p=%s), nafn(%d)\n",
+	it, nback, nback2, nforw, it->trans_parent, it->trans_parent?it->trans_parent->vtable->name:"", nafn));
     it->state &= ~PDL_DESTROYING;
     return PDL_err;
 }
 
+pdl_error pdl_destroy(pdl *it) {
+  return pdl__destroy_recprotect(it, 0);
+}
 
 /* Straight copy, no dataflow */
 pdl *pdl_hard_copy(pdl *src) {
@@ -591,7 +584,7 @@ PDL_Anyval pdl_get_offs(pdl *it, PDL_Indx offs) {
 pdl_error pdl__addchildtrans(pdl *it,pdl_trans *trans)
 {
 	pdl_error PDL_err = {0, NULL, 0};
-	PDLDEBUG_f(printf("pdl__addchildtrans\n"));
+	PDLDEBUG_f(printf("pdl__addchildtrans add to %p trans=%s\n", it, trans->vtable?trans->vtable->name:""));
 	int i; pdl_trans_children *c = &it->trans_children;
 	do {
 	    if (c->next) { c=c->next; continue; } else {
@@ -622,17 +615,18 @@ pdl_error pdl_make_physdims(pdl *it) {
 	  return PDL_err;
 	}
 	it->state &= ~PDL_PARENTDIMSCHANGED;
-	PDLDEBUG_f(printf("make_physdims %p TRANS:\n",(void*)it);
-	    pdl_dump_trans_fixspace(it->trans_parent,3));
-	for(i=0; i<it->trans_parent->vtable->nparents; i++) {
-		PDL_RETERROR(PDL_err, pdl_make_physdims(it->trans_parent->pdls[i]));
+	pdl_trans *trans = it->trans_parent;
+	PDLDEBUG_f(printf("make_physdims %p TRANS:\n",it);
+	    pdl_dump_trans_fixspace(trans,3));
+	for(i=0; i<trans->vtable->nparents; i++) {
+		PDL_RETERROR(PDL_err, pdl_make_physdims(trans->pdls[i]));
 	}
 	/* doesn't this mean that all children of this trans have
 	   now their dims set and accordingly all those flags should
 	   be reset? Otherwise redodims will be called for them again? */
 	PDLDEBUG_f(printf("make_physdims: calling redodims %p on %p\n",
-			  (void*)(it->trans_parent),(void*)it));
-	REDODIMS(PDL_RETERROR, it->trans_parent);
+			  trans,it));
+	REDODIMS(PDL_RETERROR, trans);
 	/* why this one? will the old allocated data be freed correctly? */
 	if((c & PDL_PARENTDIMSCHANGED) && (it->state & PDL_ALLOCATED)) {
 		it->state &= ~PDL_ALLOCATED;
@@ -682,7 +676,12 @@ pdl_error pdl_make_trans_mutual(pdl_trans *trans)
   pdl_error PDL_err = {0, NULL, 0};
   PDLDEBUG_f(printf("make_trans_mutual %p\n",(void*)trans);pdl_dump_trans_fixspace(trans,3));
   pdl_transvtable *vtable = trans->vtable;
+  pdl **pdls = trans->pdls;
   PDL_Indx i, npdls=vtable->npdls, nparents=vtable->nparents;
+  PDL_Indx nchildren = npdls - nparents;
+  /* copy the converted outputs from the end-area to use as actual
+    outputs - cf type_coerce */
+  for (i=vtable->nparents; i<vtable->npdls; i++) pdls[i] = pdls[i+nchildren];
   PDL_TR_CHKMAGIC(trans);
   int pfflag=0;
   PDL_err = pdl_trans_flow_null_checks(trans, &pfflag);
@@ -692,13 +691,13 @@ pdl_error pdl_make_trans_mutual(pdl_trans *trans)
   }
   char dataflow = !!(pfflag || (trans->flags & PDL_ITRANS_DO_DATAFLOW_ANY));
   for(i=0; i<nparents; i++) {
-    pdl *parent = trans->pdls[i];
+    pdl *parent = pdls[i];
     PDL_RETERROR(PDL_err, pdl__addchildtrans(parent,trans));
     if (parent->state & PDL_DATAFLOW_F) trans->flags |= PDL_ITRANS_DO_DATAFLOW_F;
   }
   int wd[npdls];
   for(i=nparents; i<npdls; i++) {
-	pdl *child = trans->pdls[i];
+	pdl *child = pdls[i];
 	char isnull = !!(child->state & PDL_NOMYDIMS);
 	wd[i]=(isnull ? PDL_PARENTDIMSCHANGED : PDL_PARENTDATACHANGED);
 	if (dataflow) {
@@ -712,7 +711,7 @@ pdl_error pdl_make_trans_mutual(pdl_trans *trans)
 	    child->state = (child->state & ~PDL_NOMYDIMS) | PDL_MYDIMS_TRANS;
   }
   if (!dataflow)
-	PDL_ACCUMERROR(PDL_err, pdl_destroytransform(trans,1,wd));
+	PDL_ACCUMERROR(PDL_err, pdl_destroytransform(trans,1,wd,0));
   PDLDEBUG_f(printf("make_trans_mutual exit %p\n",(void*)trans));
   return PDL_err;
 } /* pdl_make_trans_mutual() */
@@ -739,15 +738,13 @@ pdl_error pdl_redodims_default(pdl_trans *trans) {
   return PDL_err;
 }
 
-pdl_error pdl_make_physical(pdl *it) {
+pdl_error pdl__make_physical_recprotect(pdl *it, int recurse_count) {
 	pdl_error PDL_err = {0, NULL, 0};
 	int i, vaffinepar=0;
-	DECL_RECURSE_GUARD;
-
+	if(recurse_count > 1000)
+	  return pdl_make_error_simple(PDL_EUSERERROR, "PDL:Internal Error: data structure recursion limit exceeded (max 1000 levels)\n\tThis could mean that you have found an infinite-recursion error in PDL, or\n\tthat you are building data structures with very long dataflow dependency\n\tchains.  You may want to try using sever() to break the dependency.\n");
 	PDLDEBUG_f(printf("make_physical %p\n",(void*)it));
         PDL_CHKMAGIC(it);
-
-	START_RECURSE_GUARD;
 	if(it->state & PDL_ALLOCATED && !(it->state & PDL_ANYCHANGED))  {
 		goto mkphys_end;
 	}
@@ -756,12 +753,11 @@ pdl_error pdl_make_physical(pdl *it) {
 		goto mkphys_end;
 	}
 	if(!it->trans_parent) {
-	        ABORT_RECURSE_GUARD;
 		return pdl_make_error_simple(PDL_EFATAL, "PDL Not physical but doesn't have parent");
 	}
 	if(it->trans_parent->flags & PDL_ITRANS_ISAFFINE) {
 		if(!PDL_VAFFOK(it))
-			PDL_RETERROR(PDL_err, pdl_make_physvaffine(it));
+			PDL_RETERROR(PDL_err, pdl__make_physvaffine_recprotect(it, recurse_count+1));
 	}
 	if(PDL_VAFFOK(it)) {
 		PDLDEBUG_f(printf("make_physical: VAFFOK\n"));
@@ -773,11 +769,11 @@ pdl_error pdl_make_physical(pdl *it) {
 	PDL_TR_CHKMAGIC(it->trans_parent);
 	for(i=0; i<it->trans_parent->vtable->nparents; i++) {
 		if(VAFFINE_FLAG_OK(it->trans_parent->vtable->per_pdl_flags,i)) {
-			PDL_RETERROR(PDL_err, pdl_make_physvaffine(it->trans_parent->pdls[i]));
+			PDL_RETERROR(PDL_err, pdl__make_physvaffine_recprotect(it->trans_parent->pdls[i], recurse_count+1));
                         /* check if any of the parents is a vaffine */
                         vaffinepar = vaffinepar || (it->trans_parent->pdls[i]->data != PDL_REPRP(it->trans_parent->pdls[i]));
                 }  else
-			PDL_RETERROR(PDL_err, pdl_make_physical(it->trans_parent->pdls[i]));
+			PDL_RETERROR(PDL_err, pdl__make_physical_recprotect(it->trans_parent->pdls[i], recurse_count+1));
 	}
         /* XXX The real question is: why do we need another call to
          * redodims if !(it->state & PDL_ALLOCATED)??????
@@ -790,13 +786,15 @@ pdl_error pdl_make_physical(pdl *it) {
 	}
 	READDATA(it->trans_parent);
 	it->state &= ~(PDL_ANYCHANGED | PDL_OPT_ANY_OK);
-
   mkphys_end:
 	PDLDEBUG_f(printf("make_physical exit %p\n",(void*)it));
-	END_RECURSE_GUARD;
 	return PDL_err;
 }
 
+pdl_error pdl_make_physical(pdl *it) {
+  return pdl__make_physical_recprotect(it, 0);
+}
+
 pdl_error pdl_changed(pdl *it, int what, int recursing)
 {
     pdl_error PDL_err = {0, NULL, 0};
@@ -856,12 +854,11 @@ pdl_error pdl_changed(pdl *it, int what, int recursing)
    this function is the right one to call in any case if you want to
    make only those physical (i.e. allocating their own data, etc) which
    have to be and leave those vaffine with updated dims, etc, that do
-   have an appropriate transformation of which they are a child
-
+   have an appropriate transformation of which they are a child.
    should probably have been called make_physcareful to point out what
    it really does
 */
-pdl_error pdl_make_physvaffine(pdl *it)
+pdl_error pdl__make_physvaffine_recprotect(pdl *it, int recurse_count)
 {
 	pdl_error PDL_err = {0, NULL, 0};
 	pdl_trans *t;
@@ -873,28 +870,22 @@ pdl_error pdl_make_physvaffine(pdl *it)
 	PDL_Indx ninced;
 	int flag;
 	int incsign;
-
 	PDLDEBUG_f(printf("make_physvaffine %p\n",(void*)it));
-
 	PDL_RETERROR(PDL_err, pdl_make_physdims(it));
-
 	PDL_Indx incsleft[it->ndims];
 	if(!it->trans_parent) {
-		PDL_RETERROR(PDL_err, pdl_make_physical(it));
+		PDL_RETERROR(PDL_err, pdl__make_physical_recprotect(it, recurse_count+1));
 		goto mkphys_vaff_end;
 	}
 	if(!(it->trans_parent->flags & PDL_ITRANS_ISAFFINE)) {
-		PDL_RETERROR(PDL_err, pdl_make_physical(it));
+		PDL_RETERROR(PDL_err, pdl__make_physical_recprotect(it, recurse_count+1));
 		goto mkphys_vaff_end;
 	}
-
 	if (!it->vafftrans || it->vafftrans->ndims < it->ndims)
 	  PDL_RETERROR(PDL_err, pdl_vafftrans_alloc(it));
-
         for(i=0; i<it->ndims; i++) {
 		it->vafftrans->incs[i] = it->dimincs[i];
 	}
-
 	flag=0;
 	it->vafftrans->offs = 0;
 	t=it->trans_parent;
@@ -907,7 +898,6 @@ pdl_error pdl_make_physvaffine(pdl *it)
 		/* For all dimensions of the childest ndarray */
 		for(i=0; i<it->ndims; i++) {
 			PDL_Indx offset_left = it->vafftrans->offs;
-
 			/* inc = the increment at the current stage */
 			inc = it->vafftrans->incs[i];
 			incsign = (inc >= 0 ? 1:-1);
@@ -954,7 +944,6 @@ pdl_error pdl_make_physvaffine(pdl *it)
 			}
 			incsleft[i] = incsign*newinc;
 		}
-
 		if(flag) break;
 		for(i=0; i<it->ndims; i++) {
 			it->vafftrans->incs[i] = incsleft[i];
@@ -976,19 +965,23 @@ pdl_error pdl_make_physvaffine(pdl *it)
 	}
 	it->vafftrans->from = current;
 	it->state |= PDL_OPT_VAFFTRANSOK;
-	PDL_RETERROR(PDL_err, pdl_make_physical(current));
-
+	PDL_RETERROR(PDL_err, pdl__make_physical_recprotect(current, recurse_count+1));
   mkphys_vaff_end:
 	PDLDEBUG_f(printf("make_physvaffine exit %p\n",(void*)it));
 	return PDL_err;
 }
 
+pdl_error pdl_make_physvaffine(pdl *it)
+{
+  return pdl__make_physvaffine_recprotect(it, 0);
+}
+
 pdl_error pdl_set_datatype(pdl *a, int datatype)
 {
     pdl_error PDL_err = {0, NULL, 0};
     PDL_RETERROR(PDL_err, pdl_make_physical(a));
     if(a->trans_parent)
-	PDL_RETERROR(PDL_err, pdl_destroytransform(a->trans_parent,1,NULL));
+	PDL_RETERROR(PDL_err, pdl_destroytransform(a->trans_parent,1,NULL,0));
     if (a->state & PDL_NOMYDIMS)
 	a->datatype = datatype;
     else
@@ -1001,30 +994,35 @@ pdl_error pdl_sever(pdl *src)
     pdl_error PDL_err = {0, NULL, 0};
     if (!src->trans_parent) return PDL_err;
     PDL_RETERROR(PDL_err, pdl_make_physvaffine(src));
-    PDL_RETERROR(PDL_err, pdl_destroytransform(src->trans_parent,1,NULL));
+    PDL_RETERROR(PDL_err, pdl_destroytransform(src->trans_parent,1,NULL,0));
     return PDL_err;
 }
 
+#define PDL_MAYBE_PROPAGATE_BADFLAG(t, newval) \
+  for( i = 0; i < (t)->vtable->npdls; i++ ) { \
+    pdl *tpdl = (t)->pdls[i]; \
+    /* make sure we propagate if changed */ \
+    if (!!newval != !!(tpdl->state & PDL_BADVAL)) \
+      pdl_propagate_badflag( tpdl, newval ); \
+  }
+
 /* newval = 1 means set flag, 0 means clear it */
 void pdl_propagate_badflag( pdl *it, int newval ) {
+    PDLDEBUG_f(printf("pdl_propagate_badflag pdl=%p newval=%d\n", it, newval));
+    PDL_Indx i;
+    if (newval)
+	it->state |=  PDL_BADVAL;
+    else
+	it->state &= ~PDL_BADVAL;
+    if (it->trans_parent)
+	PDL_MAYBE_PROPAGATE_BADFLAG(it->trans_parent, newval)
     PDL_DECL_CHILDLOOP(it)
     PDL_START_CHILDLOOP(it)
 	pdl_trans *trans = PDL_CHILDLOOP_THISCHILD(it);
-	PDL_Indx i;
-	for( i = trans->vtable->nparents; i < trans->vtable->npdls; i++ ) {
-	    pdl *child = trans->pdls[i];
-	    char need_recurse = (!!newval != !!(child->state & PDL_BADVAL));
-	    if ( newval ) {
-		child->state |=  PDL_BADVAL;
-            } else {
-		child->state &= ~PDL_BADVAL;
-	    }
-	    /* make sure we propagate to grandchildren, etc if changed */
-	    if (need_recurse)
-		pdl_propagate_badflag( child, newval );
-        } /* for: i */
+	trans->bvalflag = !!newval;
+	PDL_MAYBE_PROPAGATE_BADFLAG(trans, newval)
     PDL_END_CHILDLOOP(it)
-} /* pdl_propagate_badflag */
+}
 
 void pdl_propagate_badvalue( pdl *it ) {
     PDL_DECL_CHILDLOOP(it)
@@ -1055,7 +1053,9 @@ PDL_Anyval pdl_get_pdl_badvalue( pdl *it ) {
 }
 
 pdl_trans *pdl_create_trans(pdl_transvtable *vtable) {
-    size_t it_sz = sizeof(pdl_trans)+sizeof(pdl *)*vtable->npdls;
+    size_t it_sz = sizeof(pdl_trans)+sizeof(pdl *)*(
+      vtable->npdls + (vtable->npdls - vtable->nparents) /* outputs twice */
+    );
     pdl_trans *it = malloc(it_sz);
     if (!it) return it;
     memset(it, 0, it_sz);
@@ -1090,14 +1090,14 @@ pdl_error pdl_type_coerce(pdl_trans *trans) {
   pdl_transvtable *vtable = trans->vtable;
   pdl **pdls = trans->pdls;
   trans->__datatype = -1;
-  char parent_has_badvalue = 0;
-  PDL_Anyval parent_badvalue = {PDL_INVALID, {0}};
-  if (vtable->npdls == 2 && pdls[0]->has_badvalue
-      && (vtable->par_flags[1] & PDL_PARAM_ISCREATEALWAYS)) {
-    /* P2Child case */
-    parent_has_badvalue = 1;
-    parent_badvalue = pdls[0]->badvalue;
-  }
+  char p2child_has_badvalue = (vtable->npdls == 2 && pdls[0]->has_badvalue
+      && (vtable->par_flags[1] & PDL_PARAM_ISCREATEALWAYS));
+  PDL_Anyval parent_badvalue = p2child_has_badvalue ? pdls[0]->badvalue : (PDL_Anyval){PDL_INVALID, {0}};
+  PDL_Indx nchildren = vtable->npdls - vtable->nparents;
+  /* copy the "real" (passed-in) outputs to the end-area to use as actual
+    outputs, possibly after being converted, leaving the passed-in ones
+    alone to be picked up for use in CopyBadStatusCode */
+  for (i=vtable->nparents; i<vtable->npdls; i++) pdls[i+nchildren] = pdls[i];
   for (i=0; i<vtable->npdls; i++) {
     pdl *pdl = pdls[i];
     short flags = vtable->par_flags[i];
@@ -1133,7 +1133,7 @@ pdl_error pdl_type_coerce(pdl_trans *trans) {
     }
     if ((pdl->state & PDL_NOMYDIMS) && (!pdl->trans_parent || pdl->trans_parent == trans)) {
       pdl->badvalue = parent_badvalue;
-      pdl->has_badvalue = parent_has_badvalue;
+      pdl->has_badvalue = p2child_has_badvalue;
       pdl->datatype = new_dtype;
     } else if (new_dtype != pdl->datatype) {
       PDLDEBUG_f(printf("pdl_type_coerce (%s) pdl=%"IND_FLAG" from %d to %d\n", vtable->name, i, pdl->datatype, new_dtype));
@@ -1142,7 +1142,8 @@ pdl_error pdl_type_coerce(pdl_trans *trans) {
         return pdl_make_error(PDL_EFATAL, "%s got NULL pointer from get_convertedpdl on param %s", vtable->name, vtable->par_names[i]);
       if (pdl->datatype != new_dtype)
         return pdl_make_error_simple(PDL_EFATAL, "type not expected value after get_convertedpdl\n");
-      pdls[i] = pdl;
+      /* if type-convert output, put in end-area */
+      pdls[i + (i >= vtable->nparents ? nchildren : 0)] = pdl;
     }
   }
   return PDL_err;
