@@ -1,20 +1,17 @@
 # Old results: approx. 1.91_03: 34 secs (512, 10 iter)
-# With simply folded-out threading: 3.4 secs (10fold!)
+# With simply folded-out broadcasting: 3.4 secs (10fold!)
 
 # For 512,30iter:
 
 package PDL::Bench;
 
-use vars qw(@ISA @EXPORT $AUTOLOAD);
-
+use strict;
+use warnings;
 require Exporter;
 require DynaLoader;
 
-@ISA = qw(Exporter DynaLoader);
-# Items to export into callers namespace by default. Note: do not export
-# names by default without a very good reason. Use EXPORT_OK instead.
-# Do not simply export all your public functions/methods/constants.
-@EXPORT = qw(
+our @ISA = qw(Exporter DynaLoader);
+our @EXPORT = qw(
 	do_benchmark
 );
 
