@@ -1,21 +1,23 @@
-######################################################################
-######################################################################
-## ScaleController -- this is the module that controls 3-D window scaling
-## when you drag the mouse in the display window.
+# a very simple unsophisticated scaler that
+# takes advantage of the nice infrastructure provided by TJL
+# controls 3-D window scaling
+# when you drag the mouse in the display window.
+package PDL::Graphics::TriD::SimpleScaler;
 
-
-
-package PDL::Graphics::TriD::ScaleController;
+use strict;
+use warnings;
 use base qw/PDL::Graphics::TriD::ButtonControl/;
 use fields qw/Dist/;
 
+$PDL::Graphics::TriD::verbose //= 0;
+
 sub new {
   my($type,$win,$dist) = @_;
   
   my $this = $type->SUPER::new( $win);
 
   $this->{Dist} = $dist;				  
-  $win->add_resizecommand(sub {print "Resized window: ",join(",",@_),"\n" if $PDL::debug_trid;  $this->set_wh(@_); });
+  $win->add_resizecommand(sub {print "Resized window: ",join(",",@_),"\n" if $PDL::Graphics::TriD::verbose;  $this->set_wh(@_); });
   return $this;
 }
 
@@ -30,22 +32,10 @@ sub xy2norm {
 
 sub mouse_moved {
 	my($this,$x0,$y0,$x1,$y1) = @_;
-#	$this->{Dist} *=
 	${$this->{Dist}} *=
 	  $this->xy2fac($this->xy2norm($x0,$y0),$this->xy2norm($x1,$y1));
 }
 
-##############################################################
-#
-# a very simple unsophisticated scaler that
-# takes advantage of the nice infrastructure provided by
-# TJL
-#
-##############################################################
-package PDL::Graphics::TriD::SimpleScaler;
-
-use base qw/PDL::Graphics::TriD::ScaleController/;
-
 # x,y to distance from center
 sub xy2fac {
 	my($this,$x0,$y0,$x1,$y1) = @_;
@@ -53,7 +43,4 @@ sub xy2fac {
 	return $dy>0 ? 1+2*$dy : 1/(1-2*$dy);
 }
 
-
-
-
 1;
